function [SSE_total] = OverallBaroreflexModel(par,P_in,HR_ts_cell,t_HR_ts_cell)
%try

SSE(1,1:length(HR_ts_cell(:,1))) = 0;
SSE_total = 0;

for j = 1:length(HR_ts_cell(:,1))
    
% Parameters
Cw = 0.006;
Ro = 1.6;
S = 480;
delta_th_s = par(1);
delta_th_p = par(2);
Kne = 40/40;
K1 = 60/40;
K2 = 150/40;
K3 = 42/40;
B1 = 40/40;
B2 = 400/40;
B3 = 225; %par(3)/40;
Bwall = 1;
Tsmax = 4; %par(4);
Tpmax = 3; %par(5);
Tsmin = 0.5; %par(6);
Tpmin = 0.5; %par(7);
Gcns = 1;
Gs = 0.2; %par(8);
Gp = 0.2; %par(9);
tau_nor = 9.1;
tau_ach = 0.2;
qs = 1/tau_nor;
qp = 1/tau_ach;
tau_HR_nor = 2.1;
tau_HR_ach_s = 2.5;
c_nor_half = 1.12;
c_ach_half = 0.65;
HRo = 280; %par(10);
HRmax = 286;  %par(11);
HRmin = 230; %par(12);
delta_HRs_max = HRmax - HRo;
delta_HRp_max = HRo - HRmin;
beta = 0.1; %par(13);

for i = 1:length(HR_ts_cell{j,1})
    HR_ts(i) = HR_ts_cell{j,1}{i};
    t_HR_ts(i) = t_HR_ts_cell{j,1}{i};
end

N_ts = length(P_in(:,j));
t_ts  = ((0:1:N_ts-1)*0.01)';

%ODE Initial Conditions
eps_1_init = 0.2042;
eps_2_init = 0.232;
eps_3_init = 0.149;
c_nor_init = 1.249;
P_init = P_in(1,j);
A_init = pi*(Ro + Cw*P_init)^2;

ICs = [A_init; eps_1_init; eps_2_init; eps_3_init; c_nor_init; 0; 0; 0];
tstart = 0;
tfinal = t_ts(end);
y0 = ICs;
Refine = 1;

%Calculate ODE DYDT
options = odeset('Refine',Refine,'RelTol',1E-4);
sol = ode23(@DYDT,[tstart tfinal],y0,options);

%Heart Wall Variables
A = deval(sol,t_HR_ts,1); %
R = sqrt(A./pi); %
eps_wall = (R - Ro)./Ro; %
eps_1 = deval(sol,t_HR_ts,2); %
eps_2 = deval(sol,t_HR_ts,3); %
eps_3 = deval(sol,t_HR_ts,4); %
delta = eps_wall - eps_1; %
nb = S*delta;
%Nervous System Variables
alpha_cns = Gcns*nb; %
alpha_s = Gs*alpha_cns; %
alpha_s_o = 59; %Gs*Gcns*S*delta_th_s; %
alpha_p = Gp*alpha_cns; %
alpha_p_o = 66.2;%Gp*Gcns*S*delta_th_p; %
Ts = Tsmin + (Tsmax - Tsmin)./(exp(alpha_s - Gs*alpha_s_o) + 1); %
Tp = Tpmin + (Tpmax - Tpmin)./(exp(-(alpha_p - Gp* alpha_p_o)) + 1); %
%Baroreceptor
c_nor = deval(sol,t_HR_ts,5); %
c_ach = deval(sol,t_HR_ts,6); 
delta_HRs = deval(sol,t_HR_ts,7);
delta_HRp_s = deval(sol,t_HR_ts,8);
delta_HRp_f = 0.75*delta_HRp_max*(c_ach.^2)./(c_ach_half^2 + c_ach.^2);
delta_HRp = delta_HRp_s + delta_HRp_f;
HRs = HRo + delta_HRs;
HRp = HRo - delta_HRp;
HR_model = HRp + (HRs - HRo).*(HRp - beta*HRmin)./(HRo - beta*HRmin);

%% Make Figures

hold on;
%figure(1);
%plot(t_ts,P_in,'k');
%title('P_in');
figure(2);
plot(t_HR_ts,eps_wall,'-r',t_HR_ts,eps_1,'-g',t_HR_ts,eps_2,'-b',t_HR_ts,eps_3,'-c',t_HR_ts,delta,'k');
title('strains');
figure(3);
plot(t_HR_ts,nb,'k');
title('nb');
figure(4);
plot(t_HR_ts,alpha_s - alpha_s_o,'k');
title('alpha_s - alpha_s_o');
figure(5);
plot(t_HR_ts,alpha_p - alpha_p_o,'k');
title('alpha_p - alpha_p_o');
figure(6);
plot(t_HR_ts,Ts,'r',t_HR_ts,Tp,'g');
title('Ts & Tp');
figure(7);
plot(t_HR_ts,c_nor,'r',t_HR_ts,c_ach,'g');
title('c_{nor} & c_{ach}');
figure(8);
plot(t_HR_ts,HRs,'r',t_HR_ts,HRp,'g');
title('HRs & HRp');
figure(9);
plot(t_HR_ts,HR_ts,'k','LineWidth',2); hold on;
plot(t_HR_ts,HR_model,'r','LineWidth',3);
hold off;

for i = 1:length(t_HR_ts)
    if t_HR_ts(i) >= 10
        IXstart = i;
        break
    end
end

HR_ts = HR_ts(IXstart:end);
HR = HR_model(IXstart:end);

%% calculate error and clear workspace
    SSE(1,j) = sum(((HR_ts - HR).^2)./((1).^2))/length(HR_ts);
    clear HR_ts t_HR_ts 
end

SSE_total = sum(SSE);

%% Baroreceptor model
function [dydt]= DYDT(t,y)
    P = interp1q(t_ts,P_in,t);
    A = y(1);
    eps_1 = y(2);
    eps_2 = y(3);
    eps_3 = y(4);
    c_nor = y(5);
    c_ach = y(6);
    delta_HRs = y(7);
    delta_HRp_s = y(8);
    
    R = sqrt(A/pi); %
    eps_wall = (R - Ro)/Ro; %
    delta = eps_wall - eps_1; %
    nb = S*delta; %
    alpha_cns = Gcns*nb; %
    alpha_s = Gs*alpha_cns; %
alpha_s_o = 59; %Gs*Gcns*S*delta_th_s; %
alpha_p = Gp*alpha_cns; %
alpha_p_o = 66.2;%Gp*Gcns*S*delta_th_p; %
    Ts = Tsmin + (Tsmax - Tsmin)/(exp(alpha_s - Gs*alpha_s_o) + 1); %
    Tp = Tpmin + (Tpmax - Tpmin)/(exp(-(alpha_p - Gp*alpha_p_o)) + 1); %
    
    C1 = (Kne*(eps_wall - eps_1) - K1*(eps_1 - eps_2))/B1; %
    C2 = (K1*(eps_1 - eps_2) - K2*(eps_2 - eps_3))/(B1 + B2); %
    C3 = B1/(B1 + B2); %
    C4 = B2/(B1 + B2); %
    C5 = (K2*(eps_2 - eps_3) - K3*eps_3)/(B2 + B3); %
    C6 = B2/(B2 + B3); %
    dydt(1,1) = (-((sqrt(A/pi) - Ro)/Cw) + P)/Bwall; %
    dydt(2,1) = (C1*(1 - C4*C6) + C2 + C4*C5)/(1 - C3 - C4*C6); %
    dydt(3,1) = (C2 + C4*C5 + C3*dydt(2,1))/(1 - C4*C6); %
    dydt(4,1) = C5 + C6*dydt(3,1); %
    dydt(5,1) = -c_nor/tau_nor + qs*Ts; %
    dydt(6,1) = -c_ach/tau_ach + qp*Tp;
    dydt(7,1) = (-delta_HRs + delta_HRs_max*(c_nor^2)/(c_nor_half^2 + c_nor^2))/tau_HR_nor;
    dydt(8,1) = (-delta_HRp_s + 0.25*delta_HRp_max*(c_ach^2)/(c_ach_half^2 + c_ach^2))/tau_HR_ach_s;
end

end